/*************************************************
*                                                *
*  EasyBMP Cross-Platform Windows Bitmap Library * 
*                                                *
*  Author: Paul Macklin                          *
*   email: pmacklin@math.uci.edu                 *
*                                                *
*    file: EasyBMP_VariousBMPutilities.h         *
*    date: 5-2-2005                              *
* version: 0.61                                  *
*                                                *
*   License: LGPL v 2.2                          *
* Copyright: 2005 by the EasyBMP Project         * 
*                                                *
* description: Various utilities.                *
*                                                *
*************************************************/

void GetBitmapInfo( char* szFileNameIn )
{
 FILE* fp;
 fp = fopen( szFileNameIn,"rb");
 
 if( !fp  )
 {
  cout << "Error: Cannot initialize from file " << szFileNameIn << ".\n"
       << "       File cannot be opened or does not exist.\n";
  return;
 } 
 
 WORD bfType; 
 SafeFread( (char*) &bfType , sizeof(WORD) , 1 , fp );
 DWORD bfSize; 
 SafeFread( (char*) &bfSize , sizeof(DWORD) , 1 , fp ); 
 WORD bfReserved1;
 SafeFread( (char*) &bfReserved1 , sizeof(WORD) , 1 , fp ); 
 WORD bfReserved2;
 SafeFread( (char*) &bfReserved2 , sizeof(WORD) , 1 , fp ); 
 DWORD bfOffBits;
 SafeFread( (char*) &bfOffBits , sizeof(DWORD) , 1 , fp ); 
 
 // now read the info header bits 
 DWORD biSize;
 SafeFread( (char*) &biSize , sizeof(DWORD) , 1 , fp ); 
 DWORD biWidth;
 SafeFread( (char*) &biWidth , sizeof(DWORD) , 1 , fp ); 
 DWORD biHeight;
 SafeFread( (char*) &biHeight , sizeof(DWORD) , 1 , fp ); 
 WORD biPlanes;
 SafeFread( (char*) &biPlanes , sizeof(WORD) , 1 , fp );
 WORD biBitCount;
 SafeFread( (char*) &biBitCount , sizeof(WORD) , 1 , fp );
 DWORD biCompression;
 SafeFread( (char*) &biCompression , sizeof(DWORD) , 1 , fp ); 
 DWORD biSizeImage;
 SafeFread( (char*) &biSizeImage , sizeof(DWORD) , 1 , fp ); 
 DWORD biXPelsPerMeter;
 SafeFread( (char*) &biXPelsPerMeter , sizeof(DWORD) , 1 , fp ); 
 DWORD biYPelsPerMeter;
 SafeFread( (char*) &biYPelsPerMeter , sizeof(DWORD) , 1 , fp ); 
 DWORD biClrUsed;
 SafeFread( (char*) &biClrUsed , sizeof(DWORD) , 1 , fp ); 
 DWORD biClrImportant;
 SafeFread( (char*) &biClrImportant , sizeof(DWORD) , 1 , fp ); 

 fclose( fp );

 cout << "File information for file " << szFileNameIn << ":\n\n";

 cout << "BITMAPFILEHEADER: \n"
      << "bfType: " << bfType << "\n"
      << "bfSize: " << bfSize << "\n"
      << "bfReserved1: " << bfReserved1 << "\n"
      << "bfReserved2: " << bfReserved2 << "\n"    
      << "bfOffBits: " << bfOffBits << "\n\n";

 cout << "BITMAPINFOHEADER: \n"
      << "bmih.biSize: " << biSize << "\n"
      << "bmih.biWidth: " << biWidth << "\n"
      << "bmih.biHeight: " << biHeight << "\n"
      << "bmih.biPlanes: " << biPlanes << "\n"
      << "bmih.biBitCount: " << biBitCount << "\n"
      << "bmih.biCompression: " << biCompression << "\n"
      << "bmih.biSizeImage: " << biSizeImage << "\n"
      << "bmih.biXPelsPerMeter: " << biXPelsPerMeter << "\n"
      << "bmih.biYPelsPerMeter: " << biYPelsPerMeter << "\n"
      << "bmih.biClrUsed: " << biClrUsed << "\n"
      << "bmih.biClrImportant: " << biClrImportant << "\n\n";  
 return;
}

BMIH GetBMIH( char* szFileNameIn )
{
 BMFH bmfh;
 BMIH bmih;

 FILE* fp;
 fp = fopen( szFileNameIn,"rb");

 if( !fp  )
 {
  cout << "Error: Cannot initialize from file " << szFileNameIn << ".\n"
       << "       File cannot be opened or does not exist.\n";
  return bmih;
 } 
 
 // read the bmfh, i.e., first 14 bytes (just to get it out of the way);
 
 BYTE TempBYTE;
 int i;
 for( i = 14 ; i > 0 ; i-- )
 {
  SafeFread( (char*) &TempBYTE , sizeof(BYTE) , 1, fp );
 }

 // read the bmih 

 SafeFread( (char*) &(bmih.biSize) , sizeof(DWORD) , 1 , fp );
 SafeFread( (char*) &(bmih.biWidth) , sizeof(DWORD) , 1 , fp ); 
 SafeFread( (char*) &(bmih.biHeight) , sizeof(DWORD) , 1 , fp ); 
 SafeFread( (char*) &(bmih.biPlanes) , sizeof(WORD) , 1 , fp ); 
 
 SafeFread( (char*) &(bmih.biBitCount) , sizeof(WORD) , 1 , fp );
 SafeFread( (char*) &(bmih.biCompression) , sizeof(DWORD) , 1 , fp ); 
 SafeFread( (char*) &(bmih.biSizeImage) , sizeof(DWORD) , 1 , fp ); 
 SafeFread( (char*) &(bmih.biXPelsPerMeter) , sizeof(DWORD) , 1 , fp ); 
 
 SafeFread( (char*) &(bmih.biYPelsPerMeter) , sizeof(DWORD) , 1 , fp ); 
 SafeFread( (char*) &(bmih.biClrUsed) , sizeof(DWORD) , 1 , fp ); 
 SafeFread( (char*) &(bmih.biClrImportant) , sizeof(DWORD) , 1 , fp ); 
 
 fclose( fp );

 return bmih;
}

BMFH GetBMFH( char* szFileNameIn )
{
 BMFH bmfh;

 FILE* fp;
 fp = fopen( szFileNameIn,"rb");

 
 if( !fp  )
 {
  cout << "Error: Cannot initialize from file " << szFileNameIn << ".\n"
       << "       File cannot be opened or does not exist.\n";
  bmfh.bfType = 0;
  return bmfh;
 } 
 
 SafeFread( (char*) &(bmfh.bfType) , sizeof(WORD) , 1 , fp );
 SafeFread( (char*) &(bmfh.bfSize) , sizeof(DWORD) , 1 , fp ); 
 SafeFread( (char*) &(bmfh.bfReserved1) , sizeof(WORD) , 1 , fp ); 
 SafeFread( (char*) &(bmfh.bfReserved2) , sizeof(WORD) , 1 , fp ); 
 SafeFread( (char*) &(bmfh.bfOffBits) , sizeof(DWORD) , 1 , fp ); 
 
 fclose( fp );

 return bmfh;
}

int GetBitmapColorDepth( char* szFileNameIn )
{
 BMFH bmfh = GetBMFH( szFileNameIn );
 BMIH bmih = GetBMIH( szFileNameIn );
 
 return (int) bmih.biBitCount;
}

void PixelToPixelCopy( BMP& From, int FromX, int FromY,  
                       BMP& To, int ToX, int ToY)
{
 *To(ToX,ToY) = *From(FromX,FromY);
 return;
}

void PixelToPixelCopyTransparent( BMP& From, int FromX, int FromY,  
                                  BMP& To, int ToX, int ToY,
                                  RGBApixel& Transparent )
{
 if( From(FromX,FromY)->Red != Transparent.Red ||
     From(FromX,FromY)->Green != Transparent.Green ||
     From(FromX,FromY)->Blue != Transparent.Blue )	 
 {
  *To(ToX,ToY) = *From(FromX,FromY);
 }
 return;
}

void RangedPixelToPixelCopy( BMP& From, int FromL , int FromR, int FromB, int FromT, 
                             BMP& To, int ToX, int ToY )
{
 // make sure the conventions are followed
 if( FromB < FromT )
 { int nTemp = FromT; FromT = FromB; FromB = nTemp; }

 // make sure that the copied regions exist in both bitmaps
 if( FromR >= From.TellWidth() )
 { FromR = From.TellWidth()-1; }
 if( FromT >= From.TellHeight() )
 { FromT = From.TellHeight()-1; }
 
 if( ToX+(FromR-FromL+1) >= To.TellWidth() )
 { FromR = To.TellWidth()-2+FromL-ToX; }
 if( ToY+(FromB-FromT+1) >= To.TellHeight() )
 { FromB = To.TellHeight()-2+FromT-ToY; } 

 int i,j;
 for( j=FromT ; j <= FromB ; j++ )
 { 
  for( i=FromL ; i <= FromR ; i++ )
  {
   PixelToPixelCopy( From, i,j,  
                     To, ToX+(i-FromL), ToY+(j-FromT) );
  }
 }

 return;
}

void RangedPixelToPixelCopyTransparent( 
     BMP& From, int FromL , int FromR, int FromB, int FromT, 
     BMP& To, int ToX, int ToY ,
     RGBApixel& Transparent )
{
 // make sure the conventions are followed
 if( FromB < FromT )
 { int nTemp = FromT; FromT = FromB; FromB = nTemp; }

 // make sure that the copied regions exist in both bitmaps
 if( FromR >= From.TellWidth() )
 { FromR = From.TellWidth()-1; }
 if( FromT >= From.TellHeight() )
 { FromT = From.TellHeight()-1; }
 
 if( ToX+(FromR-FromL+1) >= To.TellWidth() )
 { FromR = To.TellWidth()-2+FromL-ToX; }
 if( ToY+(FromB-FromT+1) >= To.TellHeight() )
 { FromB = To.TellHeight()-2+FromT-ToY; } 

 int i,j;
 for( j=FromT ; j <= FromB ; j++ )
 { 
  for( i=FromL ; i <= FromR ; i++ )
  {
   PixelToPixelCopyTransparent( From, i,j,  
                     To, ToX+(i-FromL), ToY+(j-FromT) , 
                     Transparent);
  }
 }

 return;
}
